@@ -1,5 +1,7 @@
 /*
  *  Copyright (C) 2002 Jorn Baayen
+ *  Copyright (C) 2003 Marco Pesenti Gritti
+ *  Copyright (C) 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,10 +17,15 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: find-dialog.c,v 1.15 2003/07/20 11:05:14 mpeseng Exp $
+ *  $Id: find-dialog.c,v 1.22 2004/03/17 09:02:31 marco Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "find-dialog.h"
+#include "ephy-file-helpers.h"
 #include "ephy-prefs.h"
 #include "ephy-embed.h"
 #include "ephy-debug.h"
@@ -42,9 +49,12 @@ void find_check_toggled_cb (GtkWidget *toggle, EphyDialog *dialog);
 
 static GObjectClass *parent_class = NULL;
 
+#define EPHY_FIND_DIALOG_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_FIND_DIALOG, FindDialogPrivate))
+
 struct FindDialogPrivate
 {
 	EphyEmbed *old_embed;
+	gboolean initialised;
 };
 
 enum
@@ -60,13 +70,13 @@ enum
 static const
 EphyDialogProperty properties [] =
 {
-	{ WINDOW_PROP, "find_dialog", NULL, PT_NORMAL, NULL },
-	{ MATCH_CASE_PROP, "case_check", CONF_FIND_MATCH_CASE, PT_NORMAL, NULL },
-	{ AUTOWRAP_PROP, "wrap_check", CONF_FIND_AUTOWRAP, PT_NORMAL, NULL },
-	{ WORD_PROP, "find_entry", CONF_FIND_WORD, PT_NORMAL, NULL },
-	{ BACK_BUTTON, "back_button", NULL, PT_NORMAL, NULL },
-	{ FORWARD_BUTTON, "forward_button", NULL, PT_NORMAL, NULL },
-	{ -1, NULL, NULL }
+	{ "find_dialog",	NULL,			PT_NORMAL, 0 },
+	{ "case_check",		CONF_FIND_MATCH_CASE,	PT_NORMAL, 0 },
+	{ "wrap_check",		CONF_FIND_AUTOWRAP,	PT_NORMAL, 0 },
+	{ "find_entry",		CONF_FIND_WORD,		PT_NORMAL, 0 },
+	{ "back_button",	NULL,			PT_NORMAL, 0 },
+	{ "forward_button",	NULL,			PT_NORMAL, 0 },
+	{ NULL }
 };
 
 GType
@@ -89,7 +99,7 @@ find_dialog_get_type (void)
                         (GInstanceInitFunc) find_dialog_init
                 };
 
-                find_dialog_type = g_type_register_static (EPHY_EMBED_DIALOG_TYPE,
+                find_dialog_type = g_type_register_static (EPHY_TYPE_EMBED_DIALOG,
 						           "FindDialog",
 						           &our_info, 0);
         }
@@ -103,24 +113,76 @@ update_navigation_controls (FindDialog *dialog, gboolean prev, gboolean next)
 {
 	GtkWidget *button;
 
-	button = ephy_dialog_get_control (EPHY_DIALOG (dialog), BACK_BUTTON);
+	button = ephy_dialog_get_control (EPHY_DIALOG (dialog),
+					  properties[BACK_BUTTON].id);
 	gtk_widget_set_sensitive (button, prev);
 
-	button = ephy_dialog_get_control (EPHY_DIALOG (dialog), FORWARD_BUTTON);
+	button = ephy_dialog_get_control (EPHY_DIALOG (dialog),
+					  properties[FORWARD_BUTTON].id);
 	gtk_widget_set_sensitive (button, next);
 }
 
+
+static void
+set_properties (FindDialog *find_dialog)
+{
+        char *search_string;
+	GValue match_case = {0, };
+	GValue wrap = {0, };
+	GValue word = {0, };
+	gboolean b_match_case;
+	gboolean b_wrap;
+	EphyDialog *dialog = EPHY_DIALOG (find_dialog);
+	EphyEmbed *embed;
+
+	if (!find_dialog->priv->initialised) return;
+
+        /* get the search string from the entry field */
+	ephy_dialog_get_value (dialog, properties[WORD_PROP].id, &word);
+        search_string = g_strdup (g_value_get_string (&word));
+	g_value_unset (&word);
+
+        /* don't do null searches */
+        if (search_string == NULL || search_string[0] == '\0')
+        {
+		update_navigation_controls (find_dialog, FALSE, FALSE);
+		g_free (search_string);
+
+                return;
+        }
+
+	ephy_dialog_get_value (dialog, properties[MATCH_CASE_PROP].id, &match_case);
+        b_match_case = g_value_get_boolean (&match_case);
+	g_value_unset (&match_case);
+
+	ephy_dialog_get_value (dialog, properties[AUTOWRAP_PROP].id, &wrap);
+        b_wrap = g_value_get_boolean (&wrap);
+	g_value_unset (&wrap);
+
+	embed = ephy_embed_dialog_get_embed (EPHY_EMBED_DIALOG(dialog));
+	g_return_if_fail (embed != NULL);
+
+        ephy_embed_find_set_properties (embed, search_string,
+					b_match_case, b_wrap);
+
+	g_free (search_string);
+}
+
 static void
 impl_show (EphyDialog *dialog)
 {
-
 	EPHY_DIALOG_CLASS (parent_class)->show (dialog);
 
+	EPHY_FIND_DIALOG (dialog)->priv->initialised = TRUE;
+
+	set_properties (EPHY_FIND_DIALOG (dialog));
+	
 	/* Focus the text entry.  This will correctly select or leave
 	 * unselected the existing text in the entry depending on the
 	 * 'gtk-entry-select-on-focus = 0 / 1' setting in user's gtkrc.
 	 */
-	gtk_widget_grab_focus (ephy_dialog_get_control (dialog, WORD_PROP));
+	gtk_widget_grab_focus (ephy_dialog_get_control
+					(dialog, properties[WORD_PROP].id));
 }
 
 static void
@@ -135,51 +197,14 @@ find_dialog_class_init (FindDialogClass *klass)
         object_class->finalize = find_dialog_finalize;
 
 	ephy_dialog_class->show = impl_show;
-}
 
-static void
-set_properties (FindDialog *find_dialog)
-{
-        char *search_string;
-	GValue match_case = {0, };
-	GValue wrap = {0, };
-	GValue word = {0, };
-	gboolean b_match_case;
-	gboolean b_wrap;
-	EphyDialog *dialog = EPHY_DIALOG (find_dialog);
-	EphyEmbed *embed;
-
-        /* get the search string from the entry field */
-	ephy_dialog_get_value (dialog, WORD_PROP, &word);
-        search_string = g_strdup (g_value_get_string (&word));
-	g_value_unset (&word);
-
-        /* don't do null searches */
-        if (search_string == NULL || search_string[0] == '\0')
-        {
-		update_navigation_controls (find_dialog, FALSE, FALSE);
-		g_free (search_string);
-
-                return;
-        }
-
-	ephy_dialog_get_value (dialog, MATCH_CASE_PROP, &match_case);
-        b_match_case = g_value_get_boolean (&match_case);
-
-	ephy_dialog_get_value (dialog, AUTOWRAP_PROP, &wrap);
-        b_wrap = g_value_get_boolean (&wrap);
-
-	embed = ephy_embed_dialog_get_embed (EPHY_EMBED_DIALOG(dialog));
-	g_return_if_fail (embed != NULL);
-
-        ephy_embed_find_set_properties (embed, search_string,
-					b_match_case, b_wrap);
+	g_type_class_add_private (object_class, sizeof (FindDialogPrivate));
 }
 
 static void
 sync_page_change (EphyEmbed *embed, const char *address, FindDialog *dialog)
 {
-	g_return_if_fail (IS_EPHY_EMBED (embed));
+	g_return_if_fail (EPHY_IS_EMBED (embed));
 
 	update_navigation_controls (dialog, TRUE, TRUE);
 	set_properties (dialog);
@@ -208,7 +233,7 @@ sync_embed (FindDialog *dialog, GParamSpec *pspec, gpointer data)
 	unset_old_embed (dialog);
 
 	embed = ephy_embed_dialog_get_embed (EPHY_EMBED_DIALOG (dialog));
-	g_return_if_fail (IS_EPHY_EMBED (embed));
+	g_return_if_fail (EPHY_IS_EMBED (embed));
 	dialog->priv->old_embed = embed;
 
 	g_signal_connect (G_OBJECT (embed), "ge_location",
@@ -227,17 +252,21 @@ find_dialog_init (FindDialog *dialog)
 	GdkPixbuf *icon;
 	GtkWidget *window;
 
-	dialog->priv = g_new0 (FindDialogPrivate, 1);
+	dialog->priv = EPHY_FIND_DIALOG_GET_PRIVATE (dialog);
 
 	dialog->priv->old_embed = NULL;
+	dialog->priv->initialised = FALSE;
 
 	ephy_dialog_construct (EPHY_DIALOG(dialog),
 			       properties,
-			       "epiphany.glade",
-			       "find_dialog");
+			       ephy_file ("epiphany.glade"),
+			       "find_dialog",
+			       NULL);
+
 	update_navigation_controls (dialog, TRUE, TRUE);
 
-	window = ephy_dialog_get_control (EPHY_DIALOG (dialog), WINDOW_PROP);
+	window = ephy_dialog_get_control (EPHY_DIALOG (dialog),
+					  properties[WINDOW_PROP].id);
 	icon = gtk_widget_render_icon (window,
 				       GTK_STOCK_FIND,
 				       GTK_ICON_SIZE_MENU,
@@ -254,19 +283,14 @@ find_dialog_finalize (GObject *object)
 {
 	FindDialog *dialog;
 
-        g_return_if_fail (object != NULL);
-        g_return_if_fail (IS_FIND_DIALOG (object));
+        g_return_if_fail (EPHY_IS_FIND_DIALOG (object));
 
-	dialog = FIND_DIALOG (object);
+	dialog = EPHY_FIND_DIALOG (object);
 
 	g_signal_handlers_disconnect_by_func (dialog, G_CALLBACK (sync_embed), NULL);
 
-        g_return_if_fail (dialog->priv != NULL);
-
 	unset_old_embed (dialog);
 
-        g_free (dialog->priv);
-
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -275,9 +299,9 @@ find_dialog_new (EphyEmbed *embed)
 {
 	FindDialog *dialog;
 
-	dialog = FIND_DIALOG (g_object_new (FIND_DIALOG_TYPE,
-				     "embed", embed,
-				     NULL));
+	dialog = EPHY_FIND_DIALOG (g_object_new (EPHY_TYPE_FIND_DIALOG,
+						 "embed", embed,
+						 NULL));
 
 	return EPHY_DIALOG(dialog);
 }
@@ -288,10 +312,10 @@ find_dialog_new_with_parent (GtkWidget *window,
 {
 	FindDialog *dialog;
 
-	dialog = FIND_DIALOG (g_object_new (FIND_DIALOG_TYPE,
-				     "embed", embed,
-				     "ParentWindow", window,
-				     NULL));
+	dialog = EPHY_FIND_DIALOG (g_object_new (EPHY_TYPE_FIND_DIALOG,
+						 "embed", embed,
+						 "parent-window", window,
+						 NULL));
 
 	return EPHY_DIALOG(dialog);
 }
@@ -299,17 +323,14 @@ find_dialog_new_with_parent (GtkWidget *window,
 static void
 find_dialog_go_next (FindDialog *dialog)
 {
-	gresult result;
 	EphyEmbed *embed;
 
-	g_return_if_fail (IS_FIND_DIALOG (dialog));
+	g_return_if_fail (EPHY_IS_FIND_DIALOG (dialog));
 
 	embed = ephy_embed_dialog_get_embed (EPHY_EMBED_DIALOG(dialog));
 	g_return_if_fail (embed != NULL);
 
-        result = ephy_embed_find_next (embed, FALSE);
-
-	if (result == G_OK)
+        if (ephy_embed_find_next (embed, FALSE))
 	{
 		update_navigation_controls (dialog, TRUE, TRUE);
 	}
@@ -322,17 +343,14 @@ find_dialog_go_next (FindDialog *dialog)
 static void
 find_dialog_go_prev (FindDialog *dialog)
 {
-	gresult result;
 	EphyEmbed *embed;
 
-	g_return_if_fail (IS_FIND_DIALOG (dialog));
+	g_return_if_fail (EPHY_IS_FIND_DIALOG (dialog));
 
 	embed = ephy_embed_dialog_get_embed (EPHY_EMBED_DIALOG(dialog));
 	g_return_if_fail (embed != NULL);
 
-	result = ephy_embed_find_next (embed, TRUE);
-
-	if (result == G_OK)
+	if (ephy_embed_find_next (embed, TRUE))
 	{
 		update_navigation_controls (dialog, TRUE, TRUE);
 	}
@@ -352,21 +370,21 @@ find_close_button_clicked_cb (GtkWidget *button,
 void find_next_button_clicked_cb (GtkWidget *button,
 				  EphyDialog *dialog)
 {
-	find_dialog_go_next (FIND_DIALOG(dialog));
+	find_dialog_go_next (EPHY_FIND_DIALOG(dialog));
 }
 
 void
 find_prev_button_clicked_cb (GtkWidget *button,
 			     EphyDialog *dialog)
 {
-	find_dialog_go_prev (FIND_DIALOG(dialog));
+	find_dialog_go_prev (EPHY_FIND_DIALOG(dialog));
 }
 
 void
 find_entry_changed_cb  (GtkWidget *editable,
 			EphyDialog *dialog)
 {
-	FindDialog *find_dialog = FIND_DIALOG(dialog);
+	FindDialog *find_dialog = EPHY_FIND_DIALOG(dialog);
 
 	update_navigation_controls (find_dialog, TRUE, TRUE);
 	set_properties (find_dialog);
@@ -376,7 +394,7 @@ void
 find_check_toggled_cb (GtkWidget *toggle,
                        EphyDialog *dialog)
 {
-	FindDialog *find_dialog = FIND_DIALOG(dialog);
+	FindDialog *find_dialog = EPHY_FIND_DIALOG(dialog);
 
 	update_navigation_controls (find_dialog, TRUE, TRUE);
 	set_properties (find_dialog);
