@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EventContext.cpp,v 1.35 2004/07/04 23:50:52 chpe Exp $
+ *  $Id: EventContext.cpp,v 1.36 2004/07/06 17:43:51 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -594,6 +594,13 @@ nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, Mozilla
 			info->type = EPHY_EMBED_EVENT_MOUSE_BUTTON3;
 		break;
 
+		case (PRUint16) -1:
+			/* when the user submits a form with Return, mozilla synthesises
+			 * a _mouse_ click event with btn=65535 (-1).
+			 */
+			info->type = EPHY_EMBED_EVENT_KEY;
+			break;
+
 		default:
 			g_warning ("Unknown mouse button");
 	}
