@@ -1,6 +1,6 @@
 /*
  *  Copyright (C) 2001 Philip Langdale
- *  Copyright (C) 2003 Christian Persch
+ *  Copyright (C) 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: FilePicker.cpp,v 1.37 2004/05/08 20:11:17 chpe Exp $
+ *  $Id: FilePicker.cpp,v 1.41 2004/06/07 14:19:54 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,7 +26,10 @@
 #include "FilePicker.h"
 #include "MozillaPrivate.h"
 
+#include "mozilla-version.h"
+
 #include <nsCOMPtr.h>
+#include <nsEmbedString.h>
 #include <nsIServiceManager.h>
 #include <nsIURI.h>
 #include <nsIFileURL.h>
@@ -35,12 +38,7 @@
 #include <nsIDOMWindow.h>
 #include <nsNetCID.h>
 
-#ifdef ALLOW_PRIVATE_STRINGS
-#include <nsString.h>
-#include <nsReadableUtils.h>
-#endif
-
-#if MOZILLA_SNAPSHOT < 18
+#if !MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
 #include <nsIDOMWindowInternal.h>
 #endif
 
@@ -58,16 +56,10 @@
 NS_IMPL_ISUPPORTS1(GFilePicker, nsIFilePicker)
 
 GFilePicker::GFilePicker()
+: mDialog(nsnull)
+, mMode(nsIFilePicker::modeOpen)
 {
 	LOG ("GFilePicker ctor (%p)", this)
-
-	mDialog = EPHY_FILE_CHOOSER (g_object_new (EPHY_TYPE_FILE_CHOOSER, NULL));
-
-	g_object_add_weak_pointer (G_OBJECT (mDialog), (gpointer *) &mDialog);
-
-	ephy_file_chooser_set_persist_key (mDialog, CONF_STATE_UPLOAD_DIR);
-
-	mMode = nsIFilePicker::modeOpen;
 }
 
 GFilePicker::~GFilePicker()
@@ -82,83 +74,70 @@ GFilePicker::~GFilePicker()
 }
 
 /* void init (in nsIDOMWindow parent, in AString title, in short mode); */
-#if MOZILLA_SNAPSHOT < 18
-NS_IMETHODIMP GFilePicker::Init(nsIDOMWindowInternal *parent, const PRUnichar *title, PRInt16 mode)
-#else
+#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
 NS_IMETHODIMP GFilePicker::Init(nsIDOMWindow *parent, const nsAString& title, PRInt16 mode)
+#else
+NS_IMETHODIMP GFilePicker::Init(nsIDOMWindowInternal *parent, const PRUnichar *title, PRInt16 mode)
 #endif
 {
 	LOG ("GFilePicker::Init")
 
-	if (parent)
-	{
-		GtkWidget *pwin = MozillaFindGtkParent (parent);
+	GtkWidget *gtkparent = MozillaFindGtkParent (parent);
+#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
+	NS_ENSURE_TRUE (gtkparent, NS_ERROR_FAILURE);
+#endif
 
-		gtk_window_set_transient_for (GTK_WINDOW (mDialog), GTK_WINDOW (pwin));
-	}
-
-#if MOZILLA_SNAPSHOT < 18
-	gtk_window_set_title (GTK_WINDOW (mDialog), NS_ConvertUTF16toUTF8 (title).get());
+	nsEmbedCString cTitle;
+#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
+	NS_UTF16ToCString (title, NS_CSTRING_ENCODING_UTF8, cTitle);
 #else
-	gtk_window_set_title (GTK_WINDOW (mDialog), NS_ConvertUCS2toUTF8(title).get());
+	NS_UTF16ToCString (nsEmbedString(title), NS_CSTRING_ENCODING_UTF8, cTitle);
 #endif
 
 	mMode = mode;
 
+	GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_OPEN;
 	switch (mode)
 	{
 		case nsIFilePicker::modeGetFolder:
-			gtk_file_chooser_set_action (GTK_FILE_CHOOSER (mDialog),
-						     GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
-			
-			gtk_dialog_add_buttons (GTK_DIALOG (mDialog),
-						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-						GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-						NULL);
-			gtk_dialog_set_default_response (GTK_DIALOG (mDialog), GTK_RESPONSE_ACCEPT);
+			action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
 			break;
-
 		case nsIFilePicker::modeOpenMultiple:
-			gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (mDialog), TRUE);
-			/* fallthrough */			
 		case nsIFilePicker::modeOpen:
-			gtk_file_chooser_set_action (GTK_FILE_CHOOSER (mDialog),
-						     GTK_FILE_CHOOSER_ACTION_OPEN);
-
-			gtk_dialog_add_buttons (GTK_DIALOG (mDialog),
-						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-						GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-						NULL);
-			gtk_dialog_set_default_response (GTK_DIALOG (mDialog), GTK_RESPONSE_ACCEPT);
-
+			action = GTK_FILE_CHOOSER_ACTION_OPEN;
 			break;
-
 		case nsIFilePicker::modeSave:
-			gtk_file_chooser_set_action (GTK_FILE_CHOOSER (mDialog),
-						     GTK_FILE_CHOOSER_ACTION_SAVE);
-	
-			gtk_dialog_add_buttons (GTK_DIALOG (mDialog),
-						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-						GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-						NULL);
-			gtk_dialog_set_default_response (GTK_DIALOG (mDialog), GTK_RESPONSE_ACCEPT);
+			action = GTK_FILE_CHOOSER_ACTION_SAVE;
 			break;
 		default:
 			g_assert_not_reached ();
 			break;
 	}
 
+	mDialog = ephy_file_chooser_new (cTitle.get(), gtkparent, action,
+					 CONF_STATE_UPLOAD_DIR,
+					 EPHY_FILE_FILTER_NONE);
+
+	if (mode == nsIFilePicker::modeOpenMultiple)
+	{
+		gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (mDialog), TRUE);
+	}
+
+	g_object_add_weak_pointer (G_OBJECT (mDialog), (gpointer *) &mDialog);
+
 	return NS_OK;
 }
 
 /* void appendFilters (in long filterMask); */
 NS_IMETHODIMP GFilePicker::AppendFilters(PRInt32 filterMask)
 {
-	// http://lxr.mozilla.org/seamonkey/source/xpfe/components/filepicker/res/locale/en-US/filepicker.properties
-	// http://lxr.mozilla.org/seamonkey/source/xpfe/components/filepicker/src/nsFilePicker.js line 131 ff
+	NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
 	LOG ("GFilePicker::AppendFilters mask=%d", filterMask)
 
+	// http://lxr.mozilla.org/seamonkey/source/xpfe/components/filepicker/res/locale/en-US/filepicker.properties
+	// http://lxr.mozilla.org/seamonkey/source/xpfe/components/filepicker/src/nsFilePicker.js line 131 ff
+
 	if (filterMask & nsIFilePicker::filterAll)
 	{
 		ephy_file_chooser_add_pattern_filter (mDialog, _("All files"),
@@ -200,20 +179,28 @@ NS_IMETHODIMP GFilePicker::AppendFilters(PRInt32 filterMask)
 }
 
 /* void appendFilter (in AString title, in AString filter); */
-#if MOZILLA_SNAPSHOT < 18
-NS_IMETHODIMP GFilePicker::AppendFilter(const PRUnichar *title, const PRUnichar *filter)
-#else
+#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
 NS_IMETHODIMP GFilePicker::AppendFilter(const nsAString& title, const nsAString& filter)
+#else
+NS_IMETHODIMP GFilePicker::AppendFilter(const PRUnichar *title, const PRUnichar *filter)
 #endif
 {
-	LOG ("GFilePicker::AppendFilter title '%s' for '%s'",
-	     NS_ConvertUTF16toUTF8 (title).get(),
-	     NS_ConvertUTF16toUTF8 (filter).get())
+	NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
-	NS_ConvertUTF16toUTF8 pattern(filter);
+	LOG ("GFilePicker::AppendFilter")
 
-	pattern.StripWhitespace();
-	if (pattern.IsEmpty()) return NS_ERROR_FAILURE;
+#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
+	if (!filter.Length()) return NS_ERROR_FAILURE;
+#else
+	if (!filter) return NS_ERROR_FAILURE;
+#endif
+
+	nsEmbedCString pattern;
+#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
+	NS_UTF16ToCString (filter, NS_CSTRING_ENCODING_UTF8, pattern);
+#else
+	NS_UTF16ToCString (nsEmbedString(filter), NS_CSTRING_ENCODING_UTF8, pattern);
+#endif
 
 	char **patterns;
 	patterns = g_strsplit (pattern.get(), ";", -1);
@@ -224,10 +211,17 @@ NS_IMETHODIMP GFilePicker::AppendFilter(const nsAString& title, const nsAString&
 
 	for (int i = 0; patterns[i] != NULL; i++)
 	{
-		gtk_file_filter_add_pattern (filth, patterns[i]);
+		gtk_file_filter_add_pattern (filth, g_strstrip (patterns[i]));
 	}
 
-	gtk_file_filter_set_name (filth, NS_ConvertUTF16toUTF8 (title).get());
+	nsEmbedCString cTitle;
+#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
+	NS_UTF16ToCString (title, NS_CSTRING_ENCODING_UTF8, cTitle);
+#else
+	NS_UTF16ToCString (nsEmbedString(title), NS_CSTRING_ENCODING_UTF8, cTitle);
+#endif
+
+	gtk_file_filter_set_name (filth, cTitle.get());
 
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (mDialog), filth);
 
@@ -237,68 +231,58 @@ NS_IMETHODIMP GFilePicker::AppendFilter(const nsAString& title, const nsAString&
 }
 
 /* attribute AString defaultString; */
-#if MOZILLA_SNAPSHOT < 18
-NS_IMETHODIMP GFilePicker::GetDefaultString(PRUnichar **aDefaultString)
-#else
+#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
 NS_IMETHODIMP GFilePicker::GetDefaultString(nsAString& aDefaultString)
+#else
+NS_IMETHODIMP GFilePicker::GetDefaultString(PRUnichar **aDefaultString)
 #endif
 {
-	char *filename, *converted;
+	NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
 	LOG ("GFilePicker::GetDefaultString")
 
-	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (mDialog));
-	if (filename != NULL)
-	{
-		converted = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
-
-#if MOZILLA_SNAPSHOT < 18
-		*aDefaultString = ToNewUnicode (NS_ConvertUTF8toUTF16 (converted));
+#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
+	aDefaultString = mDefaultString;
 #else
-		CopyUTF8toUTF16 (converted, aDefaultString);
+	*aDefaultString = NS_StringCloneData (mDefaultString);
 #endif
-	
-		g_free (filename);
-		g_free (converted);
-	}
 
 	return NS_OK;
 }
 
-#if MOZILLA_SNAPSHOT < 18
-NS_IMETHODIMP GFilePicker::SetDefaultString(const PRUnichar *aDefaultString)
-#else
+#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
 NS_IMETHODIMP GFilePicker::SetDefaultString(const nsAString& aDefaultString)
+#else
+NS_IMETHODIMP GFilePicker::SetDefaultString(const PRUnichar *aDefaultString)
 #endif
 {
-#if MOZILLA_SNAPSHOT < 18
-	LOG ("GFilePicker::SetDefaultString to %s",
-	     NS_ConvertUTF16toUTF8 (aDefaultString).get())
-#else
-	LOG ("GFilePicker::SetDefaultString to %s",
-	     NS_ConvertUCS2toUTF8 (aDefaultString).get())
-#endif
+	NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
-#if MOZILLA_SNAPSHOT < 18
-	if (aDefaultString)
-#else
-	if (aDefaultString.Length())
-#endif
+	mDefaultString.Assign (aDefaultString);
+
+	if (mMode == nsIFilePicker::modeSave)
 	{
+		nsEmbedCString defaultString;
+		NS_UTF16ToCString (mDefaultString, NS_CSTRING_ENCODING_UTF8,
+				   defaultString);
+
+		LOG ("GFilePicker::SetDefaultString %s", defaultString.get())
+
+		if (!defaultString.Length()) return NS_ERROR_FAILURE;
+
 		/* set_current_name takes UTF-8, not a filename */
 		gtk_file_chooser_set_current_name
-			(GTK_FILE_CHOOSER (mDialog),
-			 NS_ConvertUTF16toUTF8 (aDefaultString).get());
+			(GTK_FILE_CHOOSER (mDialog), defaultString.get());
 	}
 
 	return NS_OK;
 }
 
 /* attribute AString defaultExtension; */
-#if MOZILLA_SNAPSHOT < 18
-NS_IMETHODIMP GFilePicker::GetDefaultExtension(PRUnichar **aDefaultExtension)
-#else
+#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
 NS_IMETHODIMP GFilePicker::GetDefaultExtension(nsAString& aDefaultExtension)
+#else
+NS_IMETHODIMP GFilePicker::GetDefaultExtension(PRUnichar **aDefaultExtension)
 #endif
 {
 	LOG ("GFilePicker::GetDefaultExtension")
@@ -306,14 +290,13 @@ NS_IMETHODIMP GFilePicker::GetDefaultExtension(nsAString& aDefaultExtension)
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-#if MOZILLA_SNAPSHOT < 18
-NS_IMETHODIMP GFilePicker::SetDefaultExtension(const PRUnichar *aDefaultExtension)
-#else
+#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
 NS_IMETHODIMP GFilePicker::SetDefaultExtension(const nsAString& aDefaultExtension)
+#else
+NS_IMETHODIMP GFilePicker::SetDefaultExtension(const PRUnichar *aDefaultExtension)
 #endif
 {
-	LOG ("GFilePicker::SetDefaultExtension to %s",
-	     NS_ConvertUTF16toUTF8(aDefaultExtension).get())
+	LOG ("GFilePicker::SetDefaultExtension")
 
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -336,16 +319,16 @@ NS_IMETHODIMP GFilePicker::SetFilterIndex(PRInt32 aFilterIndex)
 /* attribute nsILocalFile displayDirectory; */
 NS_IMETHODIMP GFilePicker::GetDisplayDirectory(nsILocalFile **aDisplayDirectory)
 {
-	char *dir;
+	NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
 	LOG ("GFilePicker::GetDisplayDirectory")
 
-	dir = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (mDialog));
+	char *dir = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (mDialog));
 
 	if (dir != NULL)
 	{
 		nsCOMPtr<nsILocalFile> file = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-		file->InitWithNativePath (nsDependentCString (dir));
+		file->InitWithNativePath (nsEmbedCString (dir));
 		NS_IF_ADDREF (*aDisplayDirectory = file);
 	
 		g_free (dir);
@@ -356,13 +339,29 @@ NS_IMETHODIMP GFilePicker::GetDisplayDirectory(nsILocalFile **aDisplayDirectory)
 
 NS_IMETHODIMP GFilePicker::SetDisplayDirectory(nsILocalFile *aDisplayDirectory)
 {
-	nsCAutoString dir;
+	NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
+
+	nsEmbedCString dir;
 	aDisplayDirectory->GetNativePath (dir);
 
 	LOG ("GFilePicker::SetDisplayDirectory to %s", dir.get())
 
-	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (mDialog),
-					     dir.get());
+	if (mDefaultString.Length() && mMode != nsIFilePicker::modeSave)
+	{
+		nsEmbedCString defaultString;
+		NS_UTF16ToCString (mDefaultString, NS_CSTRING_ENCODING_NATIVE_FILESYSTEM,
+				   defaultString);
+
+		char *filename = g_build_filename (dir.get(), defaultString.get(), NULL);
+		LOG ("Setting filename to %s", filename);
+		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (mDialog), filename);
+		g_free (filename);
+	}
+	else
+	{
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (mDialog),
+						     dir.get());
+	}
 
 	return NS_OK;
 }
@@ -370,6 +369,8 @@ NS_IMETHODIMP GFilePicker::SetDisplayDirectory(nsILocalFile *aDisplayDirectory)
 /* readonly attribute nsILocalFile file; */
 NS_IMETHODIMP GFilePicker::GetFile(nsILocalFile **aFile)
 {
+	NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
+
 	char *filename;
 
 	LOG ("GFilePicker::GetFile")
@@ -379,7 +380,7 @@ NS_IMETHODIMP GFilePicker::GetFile(nsILocalFile **aFile)
 	if (filename != NULL)
 	{
 		nsCOMPtr<nsILocalFile> file = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-		file->InitWithNativePath (nsDependentCString (filename));
+		file->InitWithNativePath (nsEmbedCString (filename));
 		NS_IF_ADDREF (*aFile = file);
 	
 		g_free (filename);
@@ -391,6 +392,8 @@ NS_IMETHODIMP GFilePicker::GetFile(nsILocalFile **aFile)
 /* readonly attribute nsIFileURL fileURL; */
 NS_IMETHODIMP GFilePicker::GetFileURL(nsIFileURL **aFileURL)
 {
+	NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
+
 	LOG ("GFilePicker::GetFileURL")
 
 	nsCOMPtr<nsILocalFile> file;
