@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: MozRegisterComponents.cpp,v 1.21.4.1 2004/04/18 10:24:48 chpe Exp $
+ *  $Id: MozRegisterComponents.cpp,v 1.27 2004/06/03 18:51:32 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,10 +26,8 @@
 #include "ContentHandler.h"
 #include "FilePicker.h"
 #include "GlobalHistory.h"
-#include "ExternalProtocolHandlers.h"
 #include "PrintingPromptService.h"
 #include "MozDownload.h"
-#include "ExternalProtocolService.h"
 #include "EphyAboutRedirector.h"
 #include "EphyContentPolicy.h"
 
@@ -39,9 +37,8 @@
 #include "GtkNSSKeyPairDialogs.h"
 #endif
 
-#if MOZILLA_SNAPSHOT > 13
+#include <nsMemory.h>
 #include <nsDocShellCID.h>
-#endif
 #include <nsIGenericFactory.h>
 #include <nsIComponentRegistrar.h>
 #include <nsICategoryManager.h>
@@ -57,16 +54,8 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(GFilePicker)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GContentHandler)
 NS_GENERIC_FACTORY_CONSTRUCTOR(MozGlobalHistory)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GPrintingPromptService)
-NS_GENERIC_FACTORY_CONSTRUCTOR(GIRCProtocolHandler)
-NS_GENERIC_FACTORY_CONSTRUCTOR(GFtpProtocolHandler)
-NS_GENERIC_FACTORY_CONSTRUCTOR(GNewsProtocolHandler)
-NS_GENERIC_FACTORY_CONSTRUCTOR(GMailtoProtocolHandler)
 NS_GENERIC_FACTORY_CONSTRUCTOR(EphyContentPolicy)
 
-#if MOZILLA_SNAPSHOT < 12
-NS_GENERIC_FACTORY_CONSTRUCTOR(GExternalProtocolService)
-#endif
-
 #ifdef HAVE_MOZILLA_PSM
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSClientAuthDialogs)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSDialogs)
@@ -82,22 +71,15 @@ RegisterContentPolicy(nsIComponentManager *aCompMgr, nsIFile *aPath,
 		do_GetService(NS_CATEGORYMANAGER_CONTRACTID);
 	NS_ENSURE_TRUE (cm, NS_ERROR_FAILURE);
 
-	nsXPIDLCString oldval;
+	char *oldval;
 	return cm->AddCategoryEntry("content-policy",
 				    EPHY_CONTENT_POLICY_CONTRACTID,
 				    EPHY_CONTENT_POLICY_CONTRACTID,
-				    PR_TRUE, PR_TRUE, getter_Copies (oldval));
+				    PR_TRUE, PR_TRUE, &oldval);
+	nsMemory::Free (oldval);
 }
 
 static const nsModuleComponentInfo sAppComps[] = {
-#if MOZILLA_SNAPSHOT < 12
-	{
-		G_EXTERNALPROTOCOLSERVICE_CLASSNAME,
-		G_EXTERNALPROTOCOLSERVICE_CID,
-		NS_EXTERNALPROTOCOLSERVICE_CONTRACTID,
- 		GExternalProtocolServiceConstructor
-	},
-#endif
 	{
 		MOZ_DOWNLOAD_CLASSNAME,
 		MOZ_DOWNLOAD_CID,
@@ -145,11 +127,7 @@ static const nsModuleComponentInfo sAppComps[] = {
 	{
 		EPHY_GLOBALHISTORY_CLASSNAME,
 		EPHY_GLOBALHISTORY_CID,
-#if MOZILLA_SNAPSHOT > 13
 		NS_GLOBALHISTORY2_CONTRACTID,
-#else
-		NS_GLOBALHISTORY_CONTRACTID,
-#endif
 		MozGlobalHistoryConstructor
 	},
 	{
@@ -158,36 +136,6 @@ static const nsModuleComponentInfo sAppComps[] = {
 		G_PRINTINGPROMPTSERVICE_CONTRACTID,
 		GPrintingPromptServiceConstructor
 	},
-	{
-		G_IRC_PROTOCOL_CLASSNAME,
-		G_IRC_PROTOCOL_CID,
-		G_IRC_PROTOCOL_CONTRACTID,
-		GIRCProtocolHandlerConstructor
-	},
-	{
-		G_IRC_CONTENT_CLASSNAME,
-		G_IRC_PROTOCOL_CID,
-		G_IRC_CONTENT_CONTRACTID,
-		GIRCProtocolHandlerConstructor
-	},
-	{
-		G_NEWS_PROTOCOL_CLASSNAME,
-		G_NEWS_PROTOCOL_CID,
-		G_NEWS_PROTOCOL_CONTRACTID,
-		GNewsProtocolHandlerConstructor
-	},
-	{
-		G_NEWS_CONTENT_CLASSNAME,
-		G_NEWS_PROTOCOL_CID,
-		G_NEWS_CONTENT_CONTRACTID,
-		GNewsProtocolHandlerConstructor
-	},
-	{
-		G_FTP_CONTENT_CLASSNAME,
-		G_FTP_PROTOCOL_CID,
-		G_FTP_CONTENT_CONTRACTID,
-		GFtpProtocolHandlerConstructor
-	},
 	{
 		EPHY_ABOUT_REDIRECTOR_CLASSNAME,
 		EPHY_ABOUT_REDIRECTOR_CID,
@@ -206,18 +154,6 @@ static const nsModuleComponentInfo sAppComps[] = {
 		EPHY_ABOUT_REDIRECTOR_MARCO_CONTRACTID,
 		EphyAboutRedirectorConstructor
 	},
-	{
-		G_MAILTO_PROTOCOL_CLASSNAME,
-		G_MAILTO_PROTOCOL_CID,
-		G_MAILTO_PROTOCOL_CONTRACTID,
-		GMailtoProtocolHandlerConstructor
-	},
-	{
-		G_MAILTO_CONTENT_CLASSNAME,
-		G_MAILTO_PROTOCOL_CID,
-		G_MAILTO_CONTENT_CONTRACTID,
-		GMailtoProtocolHandlerConstructor
-	},
 	{
 		EPHY_CONTENT_POLICY_CLASSNAME,
 		EPHY_CONTENT_POLICY_CID,
@@ -227,18 +163,6 @@ static const nsModuleComponentInfo sAppComps[] = {
 	},
 };
 
-static const nsModuleComponentInfo sFtpComps = {
-	G_FTP_PROTOCOL_CLASSNAME,
-	G_FTP_PROTOCOL_CID,
-	G_FTP_PROTOCOL_CONTRACTID,
-	GFtpProtocolHandlerConstructor
-};
-
-static NS_DEFINE_CID(knsFtpProtocolHandlerCID, NS_FTPPROTOCOLHANDLER_CID);
-
-/* Annoying globals to track the mozilla ftp handler so it can be restored. */
-static PRBool ftpRegistered = PR_FALSE;
-static nsCOMPtr<nsIFactory> nsFtpFactory;
 
 gboolean
 mozilla_register_components (void)
@@ -293,64 +217,4 @@ mozilla_register_components (void)
 	return ret;
 }
 
-/**
- * mozilla_register_FtpProtocolHandler: Register Ftp Protocol Handler
- */
-gboolean
-mozilla_register_FtpProtocolHandler (void)
-{
-	if (ftpRegistered == PR_TRUE) return TRUE;
 
-	nsresult rv;
-
-	nsCOMPtr<nsIComponentManager> cm;
-	NS_GetComponentManager(getter_AddRefs(cm));
-	NS_ENSURE_TRUE (cm, FALSE);
-
-	rv = cm->GetClassObject(knsFtpProtocolHandlerCID,
-				NS_GET_IID(nsIFactory),
-				getter_AddRefs(nsFtpFactory));
-	if (NS_FAILED (rv)) return FALSE;
-
-	nsCOMPtr<nsIGenericFactory> ftpFactory;
-	NS_NewGenericFactory(getter_AddRefs(ftpFactory), &sFtpComps);
-	NS_ENSURE_TRUE (ftpFactory, FALSE);
-
-	nsCOMPtr<nsIComponentRegistrar> cr;
-	NS_GetComponentRegistrar(getter_AddRefs(cr));
-	NS_ENSURE_TRUE (cr, FALSE);
-
-	rv = cr->RegisterFactory(sFtpComps.mCID,
-				 sFtpComps.mDescription,
-				 sFtpComps.mContractID,
-				 ftpFactory);  
-	NS_ENSURE_SUCCESS (rv, FALSE);
-
-	ftpRegistered = PR_TRUE;
-
-	return TRUE;
-}
-
-/**
- * mozilla_unregister_FtpProtocolHandler: Unregister Ftp Protocol Handler
- */
-gboolean
-mozilla_unregister_FtpProtocolHandler (void)
-{
-	if (ftpRegistered == PR_FALSE) return FALSE;
-        
-        nsresult rv;
-	nsCOMPtr<nsIComponentRegistrar> cr;
-	NS_GetComponentRegistrar(getter_AddRefs(cr));
-	NS_ENSURE_TRUE (cr, FALSE);
-
-	rv = cr->RegisterFactory(knsFtpProtocolHandlerCID,
-				 NS_FTPPROTOCOLHANDLER_CLASSNAME,
-				 G_FTP_PROTOCOL_CONTRACTID,
-				 nsFtpFactory);
-	NS_ENSURE_SUCCESS (rv, FALSE);
-
-	ftpRegistered = PR_FALSE;
-
-        return TRUE;
-}
