@@ -15,25 +15,24 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: MozillaPrivate.cpp,v 1.9.4.2 2004/07/10 18:45:22 chpe Exp $
+ *  $Id: MozillaPrivate.cpp,v 1.15 2004/06/25 14:08:45 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include "ephy-embed.h"
+#include "mozilla-embed.h"
+
 #include "MozillaPrivate.h"
 
+#include <nsEmbedString.h>
 #include <nsIServiceManagerUtils.h>
 #include <nsIWindowWatcher.h>
 #include <nsIEmbeddingSiteWindow.h>
 #include <nsIWebBrowserChrome.h>
 #include <gtkmozembed.h>
-#include <nsReadableUtils.h>
-#include <nsString.h>
-
-#include "ephy-embed.h"
-#include "mozilla-embed.h"
 
 GtkWidget *MozillaFindEmbed (nsIDOMWindow *aDOMWindow)
 {
@@ -106,23 +105,44 @@ NS_METHOD MozillaCollatePrintSettings (const EmbedPrintInfo *info,
         options->SetMarginLeft (MM_TO_INCH (info->left_margin));
         options->SetMarginRight (MM_TO_INCH (info->right_margin));
 
-        options->SetPrinterName(NS_LITERAL_STRING("PostScript/default").get());
+	PRUnichar postscript[] = { 'P', 'o', 's', 't', 'S', 'c', 'r', 'i',
+				   'p', 't', '/', 'd', 'e', 'f', 'a', 'u',
+				   'l', 't', '\0' };
+        options->SetPrinterName(postscript);
 
-        options->SetHeaderStrLeft(NS_ConvertUTF8toUCS2(info->header_left_string).get());
+	nsEmbedString tmp;
 
-        options->SetHeaderStrCenter(NS_ConvertUTF8toUCS2(info->header_center_string).get());
+	NS_CStringToUTF16 (nsEmbedCString(info->header_left_string),
+			   NS_CSTRING_ENCODING_UTF8, tmp);
+        options->SetHeaderStrLeft (tmp.get());
 
-        options->SetHeaderStrRight(NS_ConvertUTF8toUCS2(info->header_right_string).get());
+	NS_CStringToUTF16 (nsEmbedCString(info->header_center_string),
+			   NS_CSTRING_ENCODING_UTF8, tmp);
+        options->SetHeaderStrCenter (tmp.get());
 
-        options->SetFooterStrLeft(NS_ConvertUTF8toUCS2(info->footer_left_string).get());
+	NS_CStringToUTF16 (nsEmbedCString(info->header_right_string),
+			   NS_CSTRING_ENCODING_UTF8, tmp);
+        options->SetHeaderStrRight (tmp.get());
 
-        options->SetFooterStrCenter(NS_ConvertUTF8toUCS2(info->footer_center_string).get());
+	NS_CStringToUTF16 (nsEmbedCString(info->footer_left_string),
+			   NS_CSTRING_ENCODING_UTF8, tmp);
+        options->SetFooterStrLeft (tmp.get());
 
-        options->SetFooterStrRight(NS_ConvertUTF8toUCS2(info->footer_right_string).get());
+	NS_CStringToUTF16 (nsEmbedCString(info->footer_center_string),
+			   NS_CSTRING_ENCODING_UTF8, tmp);
+        options->SetFooterStrCenter(tmp.get());
 
-        options->SetToFileName (NS_ConvertUTF8toUCS2(info->file).get());
+	NS_CStringToUTF16 (nsEmbedCString(info->footer_right_string),
+			   NS_CSTRING_ENCODING_UTF8, tmp);
+        options->SetFooterStrRight(tmp.get());
 
-	options->SetPrintCommand (NS_ConvertUTF8toUCS2(info->printer).get());
+	NS_CStringToUTF16 (nsEmbedCString(info->file),
+			   NS_CSTRING_ENCODING_UTF8, tmp);
+        options->SetToFileName (tmp.get());
+
+	NS_CStringToUTF16 (nsEmbedCString(info->printer),
+			   NS_CSTRING_ENCODING_UTF8, tmp);
+	options->SetPrintCommand (tmp.get());
 
 	/**
 	 * Work around a mozilla bug where paper size & orientation are ignored
@@ -141,7 +161,10 @@ NS_METHOD MozillaCollatePrintSettings (const EmbedPrintInfo *info,
 
 	/* native paper size formats. Our dialog does not support custom yet */
 	options->SetPaperSize (nsIPrintSettings::kPaperSizeNativeData);
-	options->SetPaperName (NS_ConvertUTF8toUCS2(info->paper).get());
+
+	NS_CStringToUTF16 (nsEmbedCString(info->paper),
+			   NS_CSTRING_ENCODING_UTF8, tmp);
+	options->SetPaperName (tmp.get());
 
         options->SetPrintInColor (info->print_color);
         options->SetOrientation (info->orientation);
