@@ -16,9 +16,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-event.cpp,v 1.3 2004/02/29 13:23:28 chpe Exp $
+ *  $Id: mozilla-embed-event.cpp,v 1.4 2004/03/17 09:02:32 marco Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "mozilla-embed-event.h"
 
 #include "ephy-debug.h"
