@@ -15,14 +15,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-file-helpers.h,v 1.6 2004/01/22 19:23:05 clahey Exp $
+ *  $Id: ephy-file-helpers.h,v 1.9 2004/02/21 19:58:05 chpe Exp $
  */
 
 #ifndef EPHY_FILE_HELPERS_H
 #define EPHY_FILE_HELPERS_H
 
 #include <glib.h>
-#include <libxml/tree.h>
 
 G_BEGIN_DECLS
 
@@ -34,7 +33,10 @@ void        ephy_file_helpers_init       (void);
 
 void        ephy_file_helpers_shutdown   (void);
 
-char       *ephy_file_tmp_directory	 (void);
+char       *ephy_file_downloads_dir      (void);
+
+const char *ephy_file_tmp_dir	 	 (void);
+
 char       *ephy_file_tmp_filename	 (const char *base,
 					  const char *extension);
 
