@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks-import.c,v 1.28 2004/03/22 09:26:14 marco Exp $
+ *  $Id: ephy-bookmarks-import.c,v 1.33 2004/06/26 12:15:21 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,7 +27,7 @@
 #include <libxml/HTMLtree.h>
 #include <libxml/xmlreader.h>
 #include <string.h>
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 
 #include <glib/gi18n.h>
 
@@ -53,44 +53,29 @@ typedef enum
 static EphyNode *
 bookmark_add (EphyBookmarks *bookmarks,
 	      const char *title,
-	      const char *address,
-	      const char *topic_name)
+	      const char *address)
 {
-	EphyNode *topic;
-	EphyNode *bmk;
-
-	if (ephy_bookmarks_find_bookmark (bookmarks, address)) return NULL;
-
-	bmk = ephy_bookmarks_add (bookmarks, title, address);
-
-	if (bmk == NULL) return NULL;
-
-	if (topic_name)
+	if (!ephy_bookmarks_find_bookmark (bookmarks, address))
 	{
-		topic = ephy_bookmarks_find_keyword (bookmarks, topic_name, FALSE);
-		if (topic == NULL)
-		{
-			topic = ephy_bookmarks_add_keyword (bookmarks, topic_name);
-		}
-
-		if (topic != NULL)
-		{
-			ephy_bookmarks_set_keyword (bookmarks, topic, bmk);
-		}
+		return ephy_bookmarks_add (bookmarks, title, address);
+	}
+	else
+	{
+		return NULL;
 	}
-
-	return bmk;
 }
 
 gboolean
 ephy_bookmarks_import (EphyBookmarks *bookmarks,
 		       const char *filename)
 {
-	char *type;
+	GnomeVFSURI *uri;
+	const char *type;
 
 	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_BOOKMARK_EDITING)) return FALSE;
 
-	type = gnome_vfs_get_mime_type (filename);
+	uri = gnome_vfs_uri_new (filename);
+	type = gnome_vfs_get_mime_type_common (uri);
 
 	LOG ("Importing bookmarks of type %s", type)
 
@@ -104,6 +89,10 @@ ephy_bookmarks_import (EphyBookmarks *bookmarks,
 	{
 		return ephy_bookmarks_import_xbel (bookmarks, filename);
 	}
+	else if (strcmp (type, "text/rdf") == 0)
+	{
+		return ephy_bookmarks_import_rdf (bookmarks, filename);
+	}
 	else if (strstr (filename, MOZILLA_BOOKMARKS_DIR) != NULL ||
                  strstr (filename, FIREBIRD_BOOKMARKS_DIR) != NULL ||
 		 strstr (filename, FIREFOX_BOOKMARKS_DIR) != NULL)
@@ -116,6 +105,8 @@ ephy_bookmarks_import (EphyBookmarks *bookmarks,
 		return ephy_bookmarks_import_xbel (bookmarks, filename);
 	}
 
+	gnome_vfs_uri_unref (uri);
+
 	return FALSE;
 }
 
@@ -253,7 +244,7 @@ xbel_parse_bookmark (EphyBookmarks *eb, xmlTextReaderPtr reader, EphyNode **ret_
 		title = xmlStrdup (_("Untitled"));
 	}
 
-	node = bookmark_add (eb, title, address, NULL);
+	node = bookmark_add (eb, title, address);
 	if (node == NULL)
 	{
 		/* probably a duplicate */
@@ -269,12 +260,10 @@ xbel_parse_bookmark (EphyBookmarks *eb, xmlTextReaderPtr reader, EphyNode **ret_
 }
 
 static int
-xbel_parse_folder (EphyBookmarks *eb, xmlTextReaderPtr reader, GList **ret_list)
+xbel_parse_folder (EphyBookmarks *eb, xmlTextReaderPtr reader, char *parent_folder)
 {
 	EphyXBELImporterState state = STATE_FOLDER;
-	EphyNode *keyword;
-	GList *list = NULL, *l;
-	xmlChar *title = NULL;
+	char *folder = NULL;
 	int ret;
 
 	ret = xmlTextReaderRead (reader);
@@ -293,9 +282,23 @@ xbel_parse_folder (EphyBookmarks *eb, xmlTextReaderPtr reader, GList **ret_list)
 		}
 		else if (xmlStrEqual (tag, "#text"))
 		{
-			if (state == STATE_TITLE && title == NULL)
+			if (state == STATE_TITLE && folder == NULL)
 			{
+				char *title;
+
 				title = xmlTextReaderValue (reader);
+
+				if (!parent_folder)
+				{
+					folder = g_strdup (title);
+				}
+				else
+				{
+					folder = g_strconcat (parent_folder, "/",
+							      title, NULL);
+				}
+
+				g_free (title);
 			}
 			else
 			{
@@ -304,13 +307,20 @@ xbel_parse_folder (EphyBookmarks *eb, xmlTextReaderPtr reader, GList **ret_list)
 		}
 		else if (xmlStrEqual (tag, "bookmark") && type == 1 && state == STATE_FOLDER)
 		{
-			EphyNode *node = NULL;
+			EphyNode *node = NULL, *keyword;
 
 			ret = xbel_parse_bookmark (eb, reader, &node);
 
-			if (EPHY_IS_NODE (node))
+			keyword = ephy_bookmarks_find_keyword (eb, folder, FALSE);
+			
+			if (keyword == NULL)
 			{
-				list = g_list_prepend (list, node);
+				keyword = ephy_bookmarks_add_keyword (eb, folder);
+			}
+
+			if (node != NULL && keyword != NULL)
+			{
+				ephy_bookmarks_set_keyword (eb, keyword, node);
 			}
 
 			if (ret != 1) break;
@@ -320,11 +330,7 @@ xbel_parse_folder (EphyBookmarks *eb, xmlTextReaderPtr reader, GList **ret_list)
 		{
 			if (type == XML_READER_TYPE_ELEMENT)
 			{
-				GList *sublist = NULL;
-
-				ret = xbel_parse_folder (eb, reader, &sublist);
-
-				list = g_list_concat (list, sublist);
+				ret = xbel_parse_folder (eb, reader, folder);
 				
 				if (ret != 1) break;
 			}
@@ -377,36 +383,7 @@ xbel_parse_folder (EphyBookmarks *eb, xmlTextReaderPtr reader, GList **ret_list)
 		ret = xmlTextReaderRead (reader);
 	}
 
-	/* tag all bookmarks in the list with keyword %title */
-	if (title == NULL || title[0] == '\0')
-	{
-		*ret_list = list;
-		return ret;
-	}
-
-	keyword = ephy_bookmarks_find_keyword (eb, title, FALSE);
-
-	if (keyword == NULL)
-	{
-		keyword = ephy_bookmarks_add_keyword (eb, title);
-	}
-
-	xmlFree (title);
-
-	if (keyword == NULL)
-	{
-		*ret_list = list;
-		return ret;
-	}
-
-	for (l = list; l != NULL; l = l->next)
-	{
-		EphyNode *node = (EphyNode *) l->data;
-
-		ephy_bookmarks_set_keyword (eb, keyword, node);
-	}
-
-	*ret_list = list;
+	g_free (folder);
 
 	return ret;
 }
@@ -444,12 +421,8 @@ xbel_parse_xbel (EphyBookmarks *eb, xmlTextReaderPtr reader)
 		else if (xmlStrEqual (tag, "folder") && type == XML_READER_TYPE_ELEMENT
 			 && state == STATE_XBEL)
 		{
-			GList *list = NULL;
-
 			/* this will eat the </folder> too */
-			ret = xbel_parse_folder (eb, reader, &list);
-
-			g_list_free (list);
+			ret = xbel_parse_folder (eb, reader, NULL);
 
 			if (ret != 1) break;
 		}
@@ -683,15 +656,34 @@ ns_parse_bookmark_item (GString *string)
 	return temp;
 }
 
+static char *
+folders_list_to_topic_name (GList *folders)
+{
+	GString *topic;
+	GList *l;
+
+	g_return_val_if_fail (folders != NULL, NULL);
+
+	topic = g_string_new (folders->data);
+
+	for (l = folders->next; l != NULL; l = l->next)
+	{
+		g_string_append (topic, "/");
+		g_string_append (topic, l->data);
+	}
+
+	return g_string_free (topic, FALSE);
+}
+
 gboolean
 ephy_bookmarks_import_mozilla (EphyBookmarks *bookmarks,
 			       const char *filename)
 {
 	FILE *bf;  /* bookmark file */
-	GString *name;
-	char *parsedname;
-	GString *url = g_string_new (NULL);
-	GList *folders = NULL, *l;
+	GString *name, *url;
+	char *parsedname, *topic;
+	EphyNode *keyword;
+	GList *folders = NULL;
 
 	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_BOOKMARK_EDITING)) return FALSE;
 
@@ -711,44 +703,42 @@ ephy_bookmarks_import_mozilla (EphyBookmarks *bookmarks,
 		switch (t)
 		{
 		case NS_FOLDER:
-			folders = g_list_prepend (folders, ns_parse_bookmark_item (name));
+			folders = g_list_append (folders, ns_parse_bookmark_item (name));
 			break;
 		case NS_FOLDER_END:
 			if (folders)
 			{
-				/* remove first entry */
-				g_free (folders->data);
-				folders = g_list_delete_link (folders, folders); 
+				GList *last = g_list_last (folders);
+
+				/* remove last entry */
+				g_free (last->data);
+				folders = g_list_delete_link (folders, last); 
 			}
 			break;
 		case NS_SITE:
 			parsedname = ns_parse_bookmark_item (name);
 
-			node = bookmark_add (bookmarks, parsedname, url->str, NULL);
+			node = bookmark_add (bookmarks, parsedname, url->str);
 
 			if (node == NULL)
 			{
 				node = ephy_bookmarks_find_bookmark (bookmarks, url->str);
 			}
 
-			for (l = folders; l != NULL; l = l->next)
-			{
-				char *topic = (char *) l->data;
-				EphyNode *keyword;
-
-				keyword = ephy_bookmarks_find_keyword (bookmarks, topic, FALSE);
+			topic = folders_list_to_topic_name (folders);
+			keyword = ephy_bookmarks_find_keyword (bookmarks, topic, FALSE);
 			
-				if (keyword == NULL)
-				{
-					keyword = ephy_bookmarks_add_keyword (bookmarks, topic);
-				}
-
-				if (node != NULL && keyword != NULL)
-				{
-					ephy_bookmarks_set_keyword (bookmarks, keyword, node);
-				}
+			if (keyword == NULL)
+			{
+				keyword = ephy_bookmarks_add_keyword (bookmarks, topic);
 			}
 
+			if (node != NULL && keyword != NULL)
+			{
+				ephy_bookmarks_set_keyword (bookmarks, keyword, node);
+			}
+
+			g_free (topic);
 			g_free (parsedname);
 
 			break;
@@ -876,7 +866,7 @@ parse_rdf_item (EphyBookmarks *bookmarks,
 		child = child->next;
 	}
 
-	bmk = bookmark_add (bookmarks, title, link, NULL);
+	bmk = bookmark_add (bookmarks, title, link);
 	if (bmk)
 	{
 		l = subjects;
